import os
import importlib
from platform import platform


def readRequirements():
    with open("requirements.txt", "r") as file:
        file = file.read()
        requirements = file.split('\n')
    return requirements


def reqcheq(verbose = False):
    requirements = readRequirements()
    for requirement in requirements:
        if verbose:
            print(f'Verificando requerimento: {requirement}')
        names = requirement.split()
        if len(names) > 1:
            install_name, import_name = names[1], names[0]
            try:
                installModule(install_name, import_name)
                if verbose:
                    print(f'Módulo já instalado: {requirement}')
            except Exception as error:
                print(f'Não foi possivel baixar o modulo: {requirement}')
                if verbose:
                    print(f'Erro: {error}')
        else:
            try:
                installModule(requirement, requirement)
                if verbose:
                    print(f'Módulo já instalado: {requirement}')
            except Exception as error:
                print(f'Não foi possivel baixar o modulo: {requirement}')
                if verbose:
                    print(f'Erro: {error}')


def installModule(install_name, import_name):

    system = platform().split('-')[0]

    try:
        module = importlib.import_module(import_name)
        # print(f'Modulo {import_name} ja instalado')
        return True
    except:
        print(f'Modulo nao encontrado: {import_name}')
        print(f'Tentando instalar automaticamente')
        try:
            if not system == 'Linux':
                os.system(f'pip install {install_name}')
            else:
                os.system(f'pip3 install {install_name}')
            print(f'Modulo instalado: {install_name}')
            return True
        except:
            return False

if __name__ == '__main__':
    reqcheq()



import mysql.connector


class Mysql():
    def __init__(self, auth:dict, login_table:str):
        self.login_table = login_table
        self.auth = auth
    
    def run(self, sql, dict_cursor=False, disconnect = True):
        try:
            if self.connection.is_connected():
                pass
        except:
            self.connect()
            
        cursor = self.connection.cursor(buffered=True, dictionary=dict_cursor)
        cursor.execute(sql)
        data = None
        
        if sql.split()[0].lower() == 'select':
            data = cursor.fetchall()
            
        self.connection.commit()  
        cursor.close()
        
        if disconnect:
            self.disconnect()
        return data

    def connect(self):
        ''' Try to connect to a database with auth params defined in config file '''
        try:
            self.connection = mysql.connector.connect(host=self.auth['host'],
                                                      database=self.auth['database'],
                                                      user=self.auth['user'],
                                                      password=self.auth['password'])
            if self.connection.is_connected():
                db_Info = self.connection.get_server_info()
                print("Connected to MySQL Server version ", db_Info)
                cursor = self.connection.cursor()
                cursor.execute("select database();")
                record = cursor.fetchone()
                print("You're connected to database: ", record)
                cursor.close()

                return self.connection

        except Exception as e:
            print(e)

    def disconnect(self):
        ''' Disconnect from database '''
        if self.connection.is_connected():
            self.connection.close()
            print("MySQL connection is closed")

    def fetchTable(self, table, rows = 0, where=[], reversed=None, ordered=None):
        ''' Fetch a number of rows from a table that exists in database.
        Number of rows and table defined in config file.
        if number of rows equals to 0, will try to fetch all rows.'''
        self.connect()
        if where:
            sql = f'SELECT * FROM `{table}` WHERE {where[0]} = "{where[1]}"'
            if reversed:
                sql = f'SELECT * FROM `{table}` WHERE {where[0]} = "{where[1]}" ORDER BY {reversed} DESC'
        else:
            sql = f"SELECT * FROM `{table}` WHERE 1"

        if ordered:
            sql = f'{sql} ORDER BY {ordered}'

        cursor = self.connection.cursor(buffered=True)
        cursor.execute(sql)
        if rows > 1:
            records = cursor.fetchmany(rows)
        else:
            records = cursor.fetchall()

        # print(f'Total number of rows in table: {cursor.rowcount}')
        # print(f'Rows fetched: {len(records)}')

        data = []
        for row in records:
            row = list(row)
            data.append(row)

        cursor.close()
        self.disconnect()
        return data

    def updateTable(self, table, id, column, value, id_column):
        self.connect()
        command = f'Update {table} set {column} = "{value}" where {id_column} = {id}'
        cursor = self.connection.cursor()
        cursor.execute(command)
        self.connection.commit()
        cursor.close()
        print("Record Updated successfully ")
        self.disconnect()


from datetime import datetime, timedelta, date
from burgos.mysql_handler import Mysql


class Connection():
    def __init__(self, ip, database, id):
        self.id = id
        self.ip = ip
        self.expira = datetime.now() + timedelta(minutes=15)

    def isExpired(self):
        if not datetime.now() < self.expira:
            return True

class Session():
    def __init__(self, database_auth:dict, login_table):
        self.connections = []
        self.database_auth = database_auth
        self.database = Mysql(database_auth, login_table)

    def getConnection(self, ip):
        for connection in self.connections:
            if connection.ip == ip:
                if not connection.isExpired():
                    return connection
                else:
                    self.connections.remove(connection)

    def login(self, user, password, ip):
        self.database.connect()
        cpf = None
        email = None
        try:
            cpf = int(user)
        except:
            pass
        
        if '@' in user:
            email = True
        
        if cpf:
            column = 'cpf'
        elif email:
            column = 'email'
        else:
            column = 'usuario'
        
        try:
            data = self.database.fetchTable(1, self.database.login_table, column, user)[0]
            if data:
                if password == data[2]:
                    id = data[0]

                    # check if user is already logged and update it' connection if it exists
                    is_logged = self.getConnection(ip)
                    if is_logged and is_logged.id == id:
                        self.connections.remove(is_logged)

                    self.connections.append(
                        Connection(ip, self.database, id))
                    self.database.connect()
                    return str(id)
        except Exception as error:
            print(error)
            self.database.connect()
            return None
